using System.Threading;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.Logging;
using Todo.Api.Hubs;
using Todo.Core.Application.Services.UserConnectionStore;
using Todo.Core.Interfaces.Publisher;
using Todo.Core.Interfaces.User;

namespace Todo.Api.Publishers;

public class TodoPublisher : ITodoPublisher
{
    private readonly IHubContext<TodoHub> _hubContext;
    private readonly ICurrentUserService _currentUserService;
    private readonly IUserConnectionStore _userConnectionStore;
    private readonly ILogger<TodoPublisher> _logger;

    public TodoPublisher(
        IHubContext<TodoHub> hubContext,
            ICurrentUserService currentUserService,
            IUserConnectionStore userConnectionStore,
        ILogger<TodoPublisher> logger)
    {
        _hubContext = hubContext;
        _currentUserService = currentUserService;
        _userConnectionStore = userConnectionStore;
        _logger = logger;
    }
    
    public async Task Publish(string todoId, CancellationToken cancellationToken)
    {
            var userId = _currentUserService.GetUserId() ?? throw new InvalidOperationException("Cannot proceed without user");
            var connections = await _userConnectionStore.GetConnectionsAsync(userId);
            
            await _hubContext
                .Clients
                .Clients(connections)
                .SendAsync("todoCreated", todoId , cancellationToken);
            
            _logger.LogInformation("todo created {TodoId}", todoId);
    }
}