using System.Threading;
using MediatR;
using Todo.Core.Application.Notifications.Todo;
using Todo.Core.Interfaces.Persistence;
using Todo.Core.Interfaces.User;

namespace Todo.Core.Application.Commands.Todo;

public record CreateTodoCommand(string TodoTitle, string? TodoProject) : IRequest<string>
{
    internal class Handler : IRequestHandler<CreateTodoCommand, string>
    {
        private readonly ICurrentUserService _currentUserService;
        private readonly ITodoRepository _todoRepository;
        private readonly IMediator _mediator;

        public Handler(ICurrentUserService currentUserService, ITodoRepository todoRepository, IMediator mediator)
        {
            _currentUserService = currentUserService;
            _todoRepository = todoRepository;
            _mediator = mediator;
        }

        public async Task<string> Handle(CreateTodoCommand request, CancellationToken cancellationToken)
        {
            var userId = _currentUserService.GetUserId();
            if (userId is null)
                throw new InvalidOperationException("User was not found");

            var todo = await _todoRepository.CreateTodoAsync(request.TodoTitle, request.TodoProject, userId);

            await _mediator.Publish(new TodoCreated(todo.Id), cancellationToken);

            return todo.Id;
        }
    }
}