using System.Text.Json;
using System.Text.Json.Serialization;
using System.Threading;
using MediatR;
using Microsoft.AspNetCore.SignalR;
using Todo.Core.Application.Services.UserConnectionStore;
using Todo.Core.Interfaces.Publisher;
using Todo.Core.Interfaces.User;

namespace Todo.Core.Application.Notifications.Todo;

public record TodoCreated([property: JsonPropertyName("todoId")] string TodoId) : INotification
{
    internal class Handler : INotificationHandler<TodoCreated>
    {
        private readonly ITodoPublisher _todoPublisher;

        public Handler(ITodoPublisher todoPublisher)
        {
            _todoPublisher = todoPublisher;
        }

        public async Task Handle(TodoCreated notification, CancellationToken cancellationToken)
        {
            await _todoPublisher.Publish(JsonSerializer.Serialize(notification), cancellationToken);
        }
    }
}